<?php

namespace Drupal\ai_upgrade_assistant\Service;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Cache\CacheBackendInterface;

/**
 * Service for handling batch processing of module analysis.
 */
class BatchAnalyzer {
  use StringTranslationTrait;

  /**
   * The project analyzer service.
   *
   * @var \Drupal\ai_upgrade_assistant\Service\ProjectAnalyzer
   */
  protected $projectAnalyzer;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The analysis tracker service.
   *
   * @var \Drupal\ai_upgrade_assistant\Service\AnalysisTracker
   */
  protected $analysisTracker;

  /**
   * The patch searcher service.
   *
   * @var \Drupal\ai_upgrade_assistant\Service\PatchSearcher
   */
  protected $patchSearcher;

  /**
   * The cache service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The batch size for processing.
   *
   * @var int
   */
  protected $batchSize = 5;

  /**
   * Constructs a new BatchAnalyzer object.
   *
   * @param \Drupal\ai_upgrade_assistant\Service\ProjectAnalyzer $project_analyzer
   *   The project analyzer service.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\ai_upgrade_assistant\Service\AnalysisTracker $analysis_tracker
   *   The analysis tracker service.
   * @param \Drupal\ai_upgrade_assistant\Service\PatchSearcher $patch_searcher
   *   The patch searcher service.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache service.
   */
  public function __construct(
    ProjectAnalyzer $project_analyzer,
    StateInterface $state,
    ModuleHandlerInterface $module_handler,
    AnalysisTracker $analysis_tracker,
    PatchSearcher $patch_searcher,
    CacheBackendInterface $cache
  ) {
    $this->projectAnalyzer = $project_analyzer;
    $this->state = $state;
    $this->moduleHandler = $module_handler;
    $this->analysisTracker = $analysis_tracker;
    $this->patchSearcher = $patch_searcher;
    $this->cache = $cache;
  }

  /**
   * Creates a batch for analyzing modules.
   *
   * @param array $modules
   *   Array of module names to analyze.
   *
   * @return array
   *   Batch definition array.
   */
  public function createBatch(array $modules) {
    // Clear previous analysis results.
    $this->state->delete('ai_upgrade_assistant.last_analysis_results');
    $this->cache->deleteMultiple(['ai_upgrade_assistant_analysis']);

    // Group modules into chunks for better performance.
    $chunks = array_chunk($modules, $this->batchSize);
    $operations = [];

    foreach ($chunks as $chunk) {
      $operations[] = [
        [$this, 'analyzeBatchChunk'],
        [$chunk],
      ];
    }

    return [
      'title' => $this->t('Analyzing modules'),
      'operations' => $operations,
      'finished' => [$this, 'batchFinished'],
      'progressive' => TRUE,
      'init_message' => $this->t('Starting module analysis...'),
      'progress_message' => $this->t('Analyzed @current out of @total module groups.'),
      'error_message' => $this->t('An error occurred during the analysis.'),
    ];
  }

  /**
   * Process a batch chunk of modules.
   *
   * @param array $modules
   *   Array of module names in this chunk.
   * @param array $context
   *   Batch context array.
   */
  public function analyzeBatchChunk(array $modules, array &$context) {
    if (!isset($context['results'])) {
      $context['results'] = [];
    }

    $cache_key = 'ai_upgrade_assistant_analysis';
    $cached_data = $this->cache->get($cache_key);
    $analysis_results = $cached_data ? $cached_data->data : [];

    foreach ($modules as $module) {
      try {
        // Check cache first.
        $module_cache_key = $cache_key . ':' . $module;
        $cached_module = $this->cache->get($module_cache_key);

        if ($cached_module) {
          $result = $cached_module->data;
        }
        else {
          $result = $this->projectAnalyzer->analyzeModule($module);
          // Cache the result for 1 hour.
          $this->cache->set($module_cache_key, $result, time() + 3600);
        }

        // Track the analysis.
        $this->analysisTracker->trackModuleAnalysis($module, $result);

        // Search for available patches.
        if (!empty($result['issues'])) {
          $patches = $this->patchSearcher->searchPatches($module);
          if (!empty($patches)) {
            $result['available_patches'] = $patches;
          }
        }

        $context['results'][$module] = $result;
        $analysis_results[$module] = $result;

        $context['message'] = $this->t('Analyzed module: @module', ['@module' => $module]);
      }
      catch (\Exception $e) {
        $error_result = [
          'status' => 'error',
          'message' => $e->getMessage(),
        ];
        $context['results'][$module] = $error_result;
        $analysis_results[$module] = $error_result;
        $this->analysisTracker->trackModuleAnalysis($module, $error_result);
      }
    }

    // Update the cache with new results.
    $this->cache->set($cache_key, $analysis_results);
  }

  /**
   * Batch finished callback.
   */
  public function batchFinished($success, array $results, array $operations) {
    if ($success) {
      // Store the final results.
      $this->state->set('ai_upgrade_assistant.last_analysis_results', $results);

      // Calculate and store statistics.
      $stats = $this->calculateAnalysisStats($results);
      $this->state->set('ai_upgrade_assistant.analysis_stats', $stats);

      // Clear old cached data.
      $this->cleanupCache();
    }
  }

  /**
   * Calculate statistics from analysis results.
   *
   * @param array $results
   *   Analysis results array.
   *
   * @return array
   *   Statistics array.
   */
  protected function calculateAnalysisStats(array $results) {
    $stats = [
      'total' => count($results),
      'compatible' => 0,
      'needs_update' => 0,
      'error' => 0,
      'with_patches' => 0,
    ];

    foreach ($results as $result) {
      if ($result['status'] === 'compatible') {
        $stats['compatible']++;
      }
      elseif ($result['status'] === 'needs_update') {
        $stats['needs_update']++;
      }
      elseif ($result['status'] === 'error') {
        $stats['error']++;
      }

      if (!empty($result['available_patches'])) {
        $stats['with_patches']++;
      }
    }

    return $stats;
  }

  /**
   * Clean up old cached data.
   */
  protected function cleanupCache() {
    // Clear analysis cache tags.
    $this->cache->deleteAll();
    $this->moduleHandler->invokeAll('ai_upgrade_assistant_analysis_complete');
  }

  /**
   * Set the batch size.
   *
   * @param int $size
   *   The new batch size.
   */
  public function setBatchSize($size) {
    $this->batchSize = max(1, (int) $size);
  }

}
